//******************************************************************************
// Copyright (C) 2015 Florian Feldbauer <feldbaue@kph.uni-mainz.de>
//                    - Helmholtz-Institut Mainz
//
// This file is part of devGpio
//
// devGpio is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 3 of the License, or
// (at your option) any later version.
//
// devGpio is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//
// version 1.0.0; Aug 13, 2015
//
//******************************************************************************

//_____ I N C L U D E S ________________________________________________________

// ANSI C/C++ includes
#include <cerrno>
#include <cstdio>
#include <cstdlib>
#include <cstring>
#include <exception>
#include <iostream>
#include <unistd.h>
#include <sstream>
#include <string>

// local includes
#include "devGpioManager.h"

int main( int argc, char* argv[] ) {
  
  if( argc < 2 ) {
    std::cerr << "Missing GPIO number as argument\n"
              << "Usage: " << argv[0] << " <gpio>"
              << std::endl;
    return -1;
  }

  epicsUInt32 gpio = 0;
  std::istringstream ss( argv[1] );
  ss >> gpio;
  if( 0 == gpio ) {
    std::cerr << "Invalid argument\n"
              << "Usage: " << argv[0] << " <gpio>"
              << std::endl;
    return -1;
  }

  try{
    std::cout << "Trying to export gpio " << gpio << std::endl;
    GpioManager::instance().exportPin( gpio );

    GpioManager::instance().waitForUdev( gpio );

    std::cout << "Set direction of gpio " << gpio << " to output" << std::endl;
    GpioManager::instance().setDirection( gpio, GpioManager::OUTPUT );

    for( int i = 0; i < 5; ++i ) {
      GpioManager::instance().setValue( gpio, 1 );
      sleep(1);
      GpioManager::instance().setValue( gpio, 0 );
      sleep(1);
    }
  } catch( std::exception &e ) {
    std::cerr << e.what() << std::endl;
    return -1;
  }

  return 0;
}

